# Copyright 2013-2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import socket

from requestbuilder import Arg, MutuallyExclusiveArgList
from requestbuilder.exceptions import ArgumentError

from euca2ools.commands.ec2 import EC2Request, parse_ports


class _ModifyNetworkAclEntry(EC2Request):
    DESCRIPTION = ('Modify a network ACL entry\n\nThis is not an '
                   'actual EC2 request -- see euca-create-network-acl-'
                   'entry(1) or euca-replace-network-acl-entry(1) for '
                   'something usable.')
    ARGS = [Arg('NetworkAclId', metavar='NACL',
                help='ID of the network ACL to add the entry to (required)'),
            Arg('-n', '--rule-number', dest='RuleNumber', metavar='INT',
                required=True, type=int,
                help='rule number for the new entry (required)'),
            MutuallyExclusiveArgList(
                Arg('--allow', dest='RuleAction', action='store_const',
                    const='allow',
                    help='make the new entry allow the traffic it matches'),
                Arg('--deny', dest='RuleAction', action='store_const',
                    const='deny',
                    help='make the new entry block the traffic it matches'))
            .required(),
            Arg('-r', '--cidr', dest='CidrBlock', metavar='CIDR',
                required=True,
                help='CIDR address range the entry should affect (required)'),
            Arg('-P', '--protocol', dest='Protocol', default='-1',
                help='protocol the entry should apply to (default: all)'),
            Arg('--egress', dest='Egress', action='store_true',
                help='''make the entry affect outgoing (egress) network
                traffic (default: affect incoming (ingress) traffic)'''),
            Arg('-p', '--port-range', dest='port_range', metavar='RANGE',
                route_to=None, help='''range of ports (specified as "from-to")
                or a single port number (required for tcp and udp)'''),
            Arg('-t', '--icmp-type-code', dest='icmp_type_code',
                metavar='TYPE:CODE', route_to=None, help='''ICMP type and
                code (specified as "type:code") (required for icmp)''')]

    def process_cli_args(self):
        self.process_port_cli_args()

    def configure(self):
        EC2Request.configure(self)
        if not self.params.get('Egress'):
            self.params['Egress'] = False
        proto = self.args.get('Protocol') or -1
        try:
            self.params['Protocol'] = int(proto)
        except ValueError:
            if proto.lower() == 'all':
                self.params['Protocol'] = -1
            else:
                try:
                    self.params['Protocol'] = socket.getprotobyname(proto)
                except socket.error:
                    raise ArgumentError('argument -n/--rule-number: unknown '
                                        'protocol "{0}"'.format(proto))
        from_port, to_port = parse_ports(proto, self.args.get('port_range'),
                                         self.args.get('icmp_type_code'))
        if self.params['Protocol'] == 1:  # ICMP
            self.params['Icmp.Type'] = from_port
            self.params['Icmp.Code'] = to_port
        else:
            self.params['PortRange.From'] = from_port
            self.params['PortRange.To'] = to_port

    def print_result(self, _):
        if self.args.get('Egress'):
            direction = 'egress'
        else:
            direction = 'ingress'
        protocol = self.params['Protocol']
        port_map = {-1: 'all', 1: 'icmp', 6: 'tcp', 17: 'udp', 132: 'sctp'}
        try:
            protocol = port_map.get(int(protocol), int(protocol))
        except ValueError:
            pass

        print self.tabify((
            'ENTRY', direction, self.params.get('RuleNumber'),
            self.params.get('RuleAction'), self.params.get('CidrBlock'),
            protocol,
            self.params.get('Icmp.Type') or self.params.get('PortRange.From'),
            self.params.get('Icmp.Code') or self.params.get('PortRange.To')))


class CreateNetworkAclEntry(_ModifyNetworkAclEntry):
    DESCRIPTION = 'Create a new entry in a VPC network ACL'


class ReplaceNetworkAclEntry(_ModifyNetworkAclEntry):
    DESCRIPTION = 'Replace an entry in a VPC network ACL'



# Copyright 2012-2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import socket

from requestbuilder import Arg, MutuallyExclusiveArgList
from requestbuilder.exceptions import ArgumentError

from euca2ools.commands.ec2 import EC2Request, parse_ports


class _ModifySecurityGroupRule(EC2Request):
    """
    The basis for security group-editing commands
    """

    ARGS = [Arg('group', metavar='GROUP', route_to=None,
                help='name or ID of the security group to modify (required)'),
            Arg('--egress', action='store_true', route_to=None,
                help='''[VPC only] manage an egress rule, which controls
                traffic leaving the group'''),
            Arg('-P', '--protocol', dest='IpPermissions.1.IpProtocol',
                metavar='PROTOCOL', default='tcp',
                help='''the protocol to affect (Non-VPC: tcp, udp, icmp)
                (VPC only: tcp, udp, icmp, -1/all, other protocol numbers)
                (default: tcp)'''),
            Arg('-p', '--port-range', dest='port_range', metavar='RANGE',
                route_to=None, help='''range of ports (specified as "from-to")
                or a single port number (required for tcp and udp)'''),
            Arg('-t', '--icmp-type-code', dest='icmp_type_code',
                metavar='TYPE:CODE', route_to=None, help='''ICMP type and
                code (specified as "type:code") (required for icmp)'''),
            MutuallyExclusiveArgList(
                Arg('-s', '--cidr', metavar='CIDR',
                    dest='IpPermissions.1.IpRanges.1.CidrIp',
                    help='''IP range (default: 0.0.0.0/0)'''),
                # ^ default is added by main()
                Arg('-o', dest='target_group', metavar='GROUP', route_to=None,
                    help='''[Non-VPC only] name of a security group with which
                    to affect network communication''')),
            Arg('-u', metavar='ACCOUNT',
                dest='IpPermissions.1.Groups.1.UserId',
                help='''ID of the account that owns the security group
                specified with -o''')]

    def process_cli_args(self):
        self.process_port_cli_args()

    # noinspection PyExceptionInherit
    def configure(self):
        EC2Request.configure(self)

        if self.args['group'].startswith('sg-'):
            # The check could probably be a little better, but meh.  Fix if
            # needed.
            self.params['GroupId'] = self.args['group']
        else:
            if self.args['egress']:
                raise ArgumentError('egress rules must use group IDs, not '
                                    'names')
            self.params['GroupName'] = self.args['group']

        target_group = self.args.get('target_group')
        if target_group is not None:
            if target_group.startswith('sg-'):
                # Same note as above
                self.params['IpPermissions.1.Groups.1.GroupId'] = target_group
            else:
                if self.args['egress']:
                    raise ArgumentError('argument -o: egress rules must use '
                                        'group IDs, not names')
                self.params['IpPermissions.1.Groups.1.GroupName'] = target_group

        protocol = self.args.get('IpPermissions.1.IpProtocol')
        if str(protocol).lower() in ('icmp', 'tcp', 'udp', '1', '6', '17'):
            from_port, to_port = parse_ports(
                protocol, self.args.get('port_range'),
                self.args.get('icmp_type_code'))
            self.params['IpPermissions.1.FromPort'] = from_port
            self.params['IpPermissions.1.ToPort'] = to_port
        elif str(protocol).lower() in ('all', '-1'):
            self.params['IpPermissions.1.IpProtocol'] = -1
        elif not str(protocol).isdigit():
            try:
                self.params['IpPermissions.1.IpProtocol'] = \
                    socket.getprotobyname(protocol)
            except socket.error:
                raise ArgumentError('argument -P: no such protocol: {0}'
                                    .format(protocol))

        if (not self.args.get('IpPermissions.1.IpRanges.1.GroupName') and
                not self.args.get('IpPermissions.1.IpRanges.1.CidrIp')):
            # Default rule target is the entire Internet
            self.params['IpPermissions.1.IpRanges.1.CidrIp'] = '0.0.0.0/0'
        if (self.params.get('IpPermissions.1.Groups.1.GroupName') and
                not self.args.get('IpPermissions.1.Groups.1.UserId')):
            raise ArgumentError('argument -u is required when -o names a '
                                'security group by name')

    def print_result(self, _):
        print self.tabify(['GROUP', self.args.get('group')])
        perm_str = ['PERMISSION', self.args.get('group'), 'ALLOWS',
                    self.params.get('IpPermissions.1.IpProtocol'),
                    self.params.get('IpPermissions.1.FromPort'),
                    self.params.get('IpPermissions.1.ToPort')]
        if self.params.get('IpPermissions.1.Groups.1.UserId'):
            perm_str.append('USER')
            perm_str.append(self.params.get('IpPermissions.1.Groups.1.UserId'))
        if self.params.get('IpPermissions.1.Groups.1.GroupName'):
            perm_str.append('NAME')
            perm_str.append(self.params.get(
                'IpPermissions.1.Groups.1.GroupName'))
        if self.params.get('IpPermissions.1.Groups.1.GroupId'):
            perm_str.append('ID')
            perm_str.append(self.params.get(
                'IpPermissions.1.Groups.1.GroupId'))
        if self.params.get('IpPermissions.1.IpRanges.1.CidrIp'):
            perm_str.extend(['FROM', 'CIDR'])
            perm_str.append(self.params.get(
                'IpPermissions.1.IpRanges.1.CidrIp'))
        print self.tabify(perm_str)


class AuthorizeSecurityGroupRule(_ModifySecurityGroupRule):
    DESCRIPTION = 'Add a rule to a security group that allows traffic to pass'

    @property
    def action(self):
        if self.args['egress']:
            return 'AuthorizeSecurityGroupEgress'
        else:
            return 'AuthorizeSecurityGroupIngress'


class RevokeSecurityGroupRule(_ModifySecurityGroupRule):
    DESCRIPTION = 'Remove a rule from a security group'

    @property
    def action(self):
        if self.args['egress']:
            return 'RevokeSecurityGroupEgress'
        else:
            return 'RevokeSecurityGroupIngress'



# Copyright 2009-2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import argparse

from requestbuilder import Arg

from euca2ools.commands.iam import IAMRequest, AS_ACCOUNT, arg_group
from euca2ools.commands.iam.deletegrouppolicy import DeleteGroupPolicy
from euca2ools.commands.iam.getgroup import GetGroup
from euca2ools.commands.iam.listgrouppolicies import ListGroupPolicies
from euca2ools.commands.iam.removeuserfromgroup import RemoveUserFromGroup


class DeleteGroup(IAMRequest):
    DESCRIPTION = 'Delete a group'
    ARGS = [arg_group(help='name of the group to delete (required)'),
            Arg('-r', '--recursive', action='store_true', route_to=None,
                help='''remove all user memberships and policies associated
                with the group first'''),
            Arg('-R', '--recursive-euca', dest='IsRecursive',
                action='store_const', const='true', help=argparse.SUPPRESS),
            Arg('-p', '--pretend', action='store_true', route_to=None,
                help='''list the user memberships and policies that would be
                deleted instead of actually deleting them. Implies -r.'''),
            AS_ACCOUNT]

    def main(self):
        if self.args['recursive'] or self.args['pretend']:
            # Figure out what we'd have to delete
            req = GetGroup.from_other(
                self, GroupName=self.args['GroupName'],
                DelegateAccount=self.params['DelegateAccount'])
            members = req.main().get('Users', [])
            req = ListGroupPolicies.from_other(
                self, GroupName=self.args['GroupName'],
                DelegateAccount=self.params['DelegateAccount'])
            policies = req.main().get('PolicyNames', [])
        else:
            # Just in case
            members = []
            policies = []
        if self.args['pretend']:
            return {'members':  [member['Arn'] for member in members],
                    'policies': policies}
        else:
            if self.args['recursive']:
                member_names = [member['UserName'] for member in members]
                req = RemoveUserFromGroup.from_other(
                    self, GroupName=self.args['GroupName'],
                    user_names=member_names,
                    DelegateAccount=self.params['DelegateAccount'])
                req.main()
                for policy in policies:
                    req = DeleteGroupPolicy.from_other(
                        self, GroupName=self.args['GroupName'],
                        PolicyName=policy,
                        DelegateAccount=self.params['DelegateAccount'])
                    req.main()
            return self.send()

    def print_result(self, result):
        if self.args['pretend']:
            print 'users'
            for arn in result['members']:
                print '\t' + arn
            print 'policies'
            for policy in result['policies']:
                print '\t' + policy



# Copyright 2013, 2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from euca2ools.commands.autoscaling import AutoScalingRequest
from requestbuilder import Arg
from requestbuilder.mixins import TabifyingMixin
from requestbuilder.response import PaginatedResponse


class DescribeAutoScalingGroups(AutoScalingRequest, TabifyingMixin):
    DESCRIPTION = 'Describe auto-scaling groups'
    ARGS = [Arg('AutoScalingGroupNames.member', metavar='ASGROUP',
                nargs='*',
                help='limit results to specific auto-scaling groups'),
            Arg('--show-long', action='store_true', route_to=None,
                help="show all of the groups' info")]
    LIST_TAGS = ['AutoScalingGroups', 'AvailabilityZones', 'EnabledMetrics',
                 'Instances', 'LoadBalancerNames', 'SuspendedProcesses',
                 'Tags', 'TerminationPolicies']

    def main(self):
        return PaginatedResponse(self, (None,), ('AutoScalingGroups',))

    def prepare_for_page(self, page):
        # Pages are defined by NextToken
        self.params['NextToken'] = page

    def get_next_page(self, response):
        return response.get('NextToken') or None

    def print_result(self, result):
        lines = []
        for group in result.get('AutoScalingGroups', []):
            bits = ['AUTO-SCALING-GROUP',
                    group.get('AutoScalingGroupName'),
                    group.get('LaunchConfigurationName'),
                    ','.join(group.get('AvailabilityZones'))]
            if self.args['show_long']:
                bits.append(group.get('CreatedTime'))
            balancers = group.get('LoadBalancerNames')
            if balancers:
                bits.append(','.join(balancers))
            else:
                bits.append(None)
            if self.args['show_long']:
                bits.append(group.get('HealthCheckType'))
            bits.append(group.get('MinSize'))
            bits.append(group.get('MaxSize'))
            bits.append(group.get('DesiredCapacity'))
            if self.args['show_long']:
                bits.append(group.get('DefaultCooldown'))
                bits.append(group.get('HealthCheckGracePeriod'))
                bits.append(group.get('VPCZoneIdentifier'))
                bits.append(group.get('PlacementGroup'))
                bits.append(group.get('AutoScalingGroupARN'))
            policies = group.get('TerminationPolicies')
            if policies:
                bits.append(','.join(policies))
            else:
                bits.append(None)
            lines.append(self.tabify(bits))
            for instance in group.get('Instances', []):
                lines.append(self._get_tabified_instance(instance))
            scale_group = group.get('AutoScalingGroupName')
            for process in group.get('SuspendedProcesses', []):
                lines.append(self._get_tabified_suspended_process(process,
                                                                  scale_group))
            for metric in group.get('EnabledMetrics', []):
                lines.append(self._get_tabified_metric(metric))
            for tag in group.get('Tags') or []:
                lines.append(self.tabify(['TAG', tag.get('ResourceType'),
                                          tag.get('ResourceId'),
                                          tag.get('Key'), tag.get('Value'),
                                          tag.get('PropagateAtLaunch')]))
        for line in lines:
            print line

    def _get_tabified_instance(self, instance):
        return self.tabify(['INSTANCE',
                            instance.get('InstanceId'),
                            instance.get('AvailabilityZone'),
                            instance.get('LifecycleState'),
                            instance.get('HealthStatus'),
                            instance.get('LaunchConfigurationName')])

    def _get_tabified_suspended_process(self, process, scale_group):
        return self.tabify(['SUSPENDED-PROCESS',
                            process.get('ProcessName'),
                            process.get('SuspensionReason'),
                            scale_group])

    def _get_tabified_metric(self, metric):
        return self.tabify(['ENABLED-METRICS',
                            metric.get('Metric'),
                            metric.get('Granularity')])



# Copyright 2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import tempfile

from requestbuilder import Arg, MutuallyExclusiveArgList
from requestbuilder.exceptions import ArgumentError

from euca2ools.commands.ec2 import EC2Request
from euca2ools.commands.ec2.describeconversiontasks import \
    DescribeConversionTasks
from euca2ools.commands.ec2.mixins import S3AccessMixin
from euca2ools.commands.ec2.structures import ImportManifest
from euca2ools.commands.s3.deleteobject import DeleteObject
from euca2ools.commands.s3.getobject import GetObject
from euca2ools.exceptions import AWSError


class DeleteDiskImage(EC2Request, S3AccessMixin):
    DESCRIPTION = 'Delete a disk image used for an import task'
    ARGS = [MutuallyExclusiveArgList(
                Arg('-t', '--task',
                    help='ID of the task to delete the image from'),
                Arg('-u', '--manifest-url',
                    help='location of the import manifest'))
            .required(),
            Arg('--ignore-active-task', action='store_true',
                help='''delete the image even if the import task is active
                (only works with -t/--task)''')]

    def configure(self):
        EC2Request.configure(self)
        self.configure_s3_access()
        if self.args.get('ignore_active_task') and not self.args.get('task'):
            raise ArgumentError('argument --ignore-active-task my only be '
                                'used with -t/--task')

    def main(self):
        if self.args.get('manifest_url'):
            manifest_url = self.args['manifest_url']
        if self.args.get('task'):
            desc_conv = DescribeConversionTasks.from_other(
                self, ConversionTaskId=[self.args['task']])
            task = desc_conv.main()['conversionTasks'][0]
            assert task['conversionTaskId'] == self.args['task']
            if task.get('importVolume'):
                vol_container = task['importVolume']
            else:
                vol_container = task['importInstance']['volumes'][0]
            manifest_url = vol_container['image']['importManifestUrl']
        _, bucket, key = self.args['s3_service'].resolve_url_to_location(
            manifest_url)
        manifest_s3path = '/'.join((bucket, key))
        manifest = self.__download_manifest(manifest_s3path)

        for part in manifest.image_parts:
            delete_req = DeleteObject.from_other(
                self, service=self.args['s3_service'],
                auth=self.args['s3_auth'], path='/'.join((bucket, part.key)))
            delete_req.main()
        delete_req = DeleteObject.from_other(
            self, service=self.args['s3_service'], auth=self.args['s3_auth'],
            path=manifest_s3path)
        delete_req.main()

    def __download_manifest(self, s3path):
        with tempfile.SpooledTemporaryFile(max_size=1024000) as \
                manifest_destfile:
            get_req = GetObject.from_other(
                self, service=self.args['s3_service'],
                auth=self.args['s3_auth'], source=s3path,
                dest=manifest_destfile, show_progress=False)
            try:
                get_req.main()
            except AWSError as err:
                if err.status_code == 404:
                    raise ArgumentError('import manifest "{0}" does not exist'
                                        .format(s3path))
                raise
            manifest_destfile.seek(0)
            return ImportManifest.read_from_fileobj(manifest_destfile)



# Copyright 2009-2013 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from euca2ools.commands.ec2 import EC2Request
from requestbuilder import Arg, Filter


class DescribeBundleTasks(EC2Request):
    DESCRIPTION = 'Describe current instance-bundling tasks'
    ARGS = [Arg('BundleId', metavar='BUNDLE', nargs='*',
                help='limit results to specific bundle tasks')]
    FILTERS = [Filter('bundle-id', help='bundle task ID'),
               Filter('error-code',
                      help='if the task failed, the error code returned'),
               Filter('error-message',
                      help='if the task failed, the error message returned'),
               Filter('instance-id', help='ID of the bundled instance'),
               Filter('progress', help='level of task completion, in percent'),
               Filter('s3-bucket',
                      help='bucket where the image will be stored'),
               Filter('s3-prefix', help='beginning of the bundle name'),
               Filter('start-time', help='task start time'),
               Filter('state', help='task state'),
               Filter('update-time', help='most recent task update time')]
    LIST_TAGS = ['bundleInstanceTasksSet']

    def print_result(self, result):
        for task in result.get('bundleInstanceTasksSet', []):
            self.print_bundle_task(task)



# Copyright 2013 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from euca2ools.commands.argtypes import delimited_list
from euca2ools.commands.monitoring import CloudWatchRequest
from euca2ools.commands.monitoring.argtypes import cloudwatch_dimension
from requestbuilder import Arg


class PutMetricAlarm(CloudWatchRequest):
    DESCRIPTION = 'Create or update an alarm'
    ARGS = [Arg('AlarmName', metavar='ALARM',
                help='name of the alarm (required)'),
            Arg('--comparison-operator', dest='ComparisonOperator',
                choices=('GreaterThanOrEqualToThreshold',
                         'GreaterThanThreshold', 'LessThanThreshold',
                         'LessThanOrEqualToThreshold'), required=True,
                help='''arithmetic operator with which the comparison with the
                threshold will be made (required)'''),
            Arg('--evaluation-periods', dest='EvaluationPeriods', type=int,
                metavar='COUNT', required=True, help='''number of consecutive
                periods for which the value of the metric needs to be compared
                to the threshold (required)'''),
            Arg('--metric-name', dest='MetricName', metavar='METRIC',
                required=True,
                help="name for the alarm's associated metric (required)"),
            Arg('--namespace', dest='Namespace', metavar='NAMESPACE',
                required=True,
                help="namespace for the alarm's associated metric (required)"),
            Arg('--period', dest='Period', metavar='SECONDS', type=int,
                required=True, help='''period over which the specified
                statistic is applied (required)'''),
            Arg('--statistic', dest='Statistic', required=True,
                choices=('Average', 'Maximum', 'Minimum', 'SampleCount',
                         'Sum'),
                help='statistic on which to alarm (required)'),
            Arg('--threshold', dest='Threshold', metavar='FLOAT', type=float,
                required=True,
                help='value to compare the statistic against (required)'),
            Arg('--actions-enabled', dest='ActionsEnabled',
                choices=('true', 'false'), help='''whether this alarm's actions
                should be executed when it changes state'''),
            Arg('--alarm-actions', dest='AlarmActions.member',
                metavar='ARN1,ARN2,...', type=delimited_list(','),
                help='''ARNs of SNS topics to publish to when the alarm changes
                to the ALARM state'''),
            Arg('--alarm-description', dest='AlarmDescription',
                metavar='DESCRIPTION', help='description of the alarm'),
            Arg('-d', '--dimensions', dest='Dimensions.member',
                metavar='KEY1=VALUE1,KEY2=VALUE2,...',
                type=delimited_list(',', item_type=cloudwatch_dimension),
                help="dimensions for the alarm's associated metric"),
            Arg('--insufficient-data-actions', metavar='ARN1,ARN2,...',
                dest='InsufficientDataActions.member',
                type=delimited_list(','), help='''ARNs of SNS topics to publish
                to when the alarm changes to the INSUFFICIENT_DATA state'''),
            Arg('--ok-actions', dest='OKActions.member',
                metavar='ARN1,ARN2,...', type=delimited_list(','),
                help='''ARNs of SNS topics to publish to when the alarm changes
                to the OK state'''),
            Arg('--unit', dest='Unit',
                help="unit for the alarm's associated metric")]



# Copyright 2009-2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import sys

from requestbuilder import Arg
import six

from euca2ools.commands.iam import IAMRequest, AS_ACCOUNT, arg_user
from euca2ools.commands.iam.getuser import GetUser
import euca2ools.exceptions
import euca2ools.util


class CreateAccessKey(IAMRequest):
    DESCRIPTION = 'Create a new access key for a user'
    ARGS = [arg_user(help='''user the new key will belong to
                     (default: current user)'''),
            Arg('-w', '--write-config', action='store_true', route_to=None,
                help='''output access keys and region information in the
                form of a euca2ools.ini(5) configuration file instead of
                by themselves'''),
            Arg('-d', '--domain', route_to=None, help='''the DNS domain
                to use for region information in configuration file
                output (default: based on IAM URL)'''),
            Arg('-l', '--set-default-user', action='store_true', route_to=None,
                help='''set this user as the default user for the region
                in euca2ools.ini(5) configuration file output.  This
                option is only useful when used with -w.'''),
            AS_ACCOUNT]

    def postprocess(self, result):
        if self.args.get('write_config'):
            parsed = six.moves.urllib.parse.urlparse(self.service.endpoint)
            if not self.args.get('domain'):
                dnsname = parsed.netloc.split(':')[0]
                if all(label.isdigit() for label in dnsname.split('.')):
                    msg = ('warning: IAM URL {0} refers to a specific IP; '
                           'for a complete configuration file supply '
                           'the region\'s DNS domain with -d/--domain'
                           .format(self.service.endpoint))
                    print >> sys.stderr, msg
                else:
                    self.args['domain'] = parsed.netloc.split('.', 1)[1]
            configfile = six.moves.configparser.SafeConfigParser()
            if self.args.get('domain'):
                if ':' not in self.args['domain'] and ':' in parsed.netloc:
                    # Add the port
                    self.args['domain'] += ':' + parsed.netloc.split(':')[1]
                # This uses self.config.region instead of
                # self.service.region_name because the latter is a global
                # service in AWS and thus frequently deferred with "use"
                # statements.  That may eventually happen in eucalyptus
                # cloud federations as well.
                #
                # At some point an option that lets one choose a region
                # name at the command line may be useful, but until
                # someone asks for it let's not clutter it up for now.
                region_name = self.config.region or self.args['domain']
                region_section = 'region {0}'.format(region_name.split(':')[0])
                configfile.add_section(region_section)
                for service in sorted(euca2ools.util.generate_service_names()):
                    url = '{scheme}://{service}.{domain}/'.format(
                        scheme=parsed.scheme, domain=self.args['domain'],
                        service=service)
                    configfile.set(region_section, '{0}-url'.format(service),
                                   url)

            user_name = result['AccessKey'].get('UserName') or 'root'
            account_id = self.get_user_account_id()
            if account_id:
                user_name = '{0}:{1}'.format(account_id, user_name)
            user_section = 'user {0}'.format(user_name)
            configfile.add_section(user_section)
            configfile.set(user_section, 'key-id',
                           result['AccessKey']['AccessKeyId'])
            configfile.set(user_section, 'secret-key',
                           result['AccessKey']['SecretAccessKey'])
            if account_id:
                configfile.set(user_section, 'account-id', account_id)
            if self.args.get('set_default_user'):
                configfile.set(region_section, 'user', user_name)
            result['configfile'] = configfile

    def print_result(self, result):
        if self.args.get('write_config'):
            result['configfile'].write(sys.stdout)
        else:
            print result['AccessKey']['AccessKeyId']
            print result['AccessKey']['SecretAccessKey']

    def get_user_account_id(self):
        req = GetUser.from_other(
            self, UserName=self.params['UserName'],
            DelegateAccount=self.params.get('DelegateAccount'))
        try:
            response = req.main()
        except euca2ools.exceptions.AWSError as err:
            if err.status_code == 403:
                msg = ('warning: unable to retrieve account ID ({0})'
                       .format(err.message))
                print >> sys.stderr, msg
                return None
            raise
        arn = response['User']['Arn']
        return arn.split(':')[4]



# Copyright 2013-2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import hashlib
import os.path
import sys

from requestbuilder import Arg
from requestbuilder.exceptions import ArgumentError
from requestbuilder.mixins import FileTransferProgressBarMixin

from euca2ools.commands.s3 import S3Request
import euca2ools.bundle.pipes


class GetObject(S3Request, FileTransferProgressBarMixin):
    DESCRIPTION = 'Retrieve objects from the server'
    ARGS = [Arg('source', metavar='BUCKET/KEY', route_to=None,
                help='the object to download (required)'),
            Arg('-o', dest='dest', metavar='PATH', route_to=None,
                default='.', help='''where to download to.  If this names a
                directory the object will be written to a file inside of that
                directory.  If this is is "-" the object will be written to
                stdout.  Otherwise it will be written to a file with the name
                given.  (default:  current directory)''')]

    def configure(self):
        S3Request.configure(self)

        bucket, _, key = self.args['source'].partition('/')
        if not bucket:
            raise ArgumentError('source must contain a bucket name')
        if not key:
            raise ArgumentError('source must contain a key name')

        if isinstance(self.args.get('dest'), basestring):
            # If it is not a string we assume it is a file-like object
            if self.args['dest'] == '-':
                self.args['dest'] = sys.stdout
            elif os.path.isdir(self.args['dest']):
                basename = os.path.basename(key)
                if not basename:
                    raise ArgumentError("specify a complete file path with -o "
                                        "to download objects that end in '/'")
                dest_path = os.path.join(self.args['dest'], basename)
                self.args['dest'] = open(dest_path, 'w')
            else:
                self.args['dest'] = open(self.args['dest'], 'w')

    def preprocess(self):
        self.path = self.args['source']

    def main(self):
        # Note that this method does not close self.args['dest']
        self.preprocess()
        bytes_written = 0
        md5_digest = hashlib.md5()
        sha_digest = hashlib.sha1()
        response = self.send()
        content_length = response.headers.get('Content-Length')
        if content_length:
            pbar = self.get_progressbar(label=self.args['source'],
                                        maxval=int(content_length))
        else:
            pbar = self.get_progressbar(label=self.args['source'])
        pbar.start()
        for chunk in response.iter_content(chunk_size=euca2ools.BUFSIZE):
            self.args['dest'].write(chunk)
            bytes_written += len(chunk)
            md5_digest.update(chunk)
            sha_digest.update(chunk)
            if pbar is not None:
                pbar.update(bytes_written)
        self.args['dest'].flush()
        pbar.finish()

        # Integrity checks
        if content_length and bytes_written != int(content_length):
            self.log.error('rejecting download due to Content-Length size '
                           'mismatch (expected: %i, actual: %i)',
                           content_length, bytes_written)
            raise RuntimeError('downloaded file appears to be corrupt '
                               '(expected size: {0}, actual: {1})'
                               .format(content_length, bytes_written))
        etag = response.headers.get('ETag', '').lower().strip('"')
        if (len(etag) == 32 and
                all(char in '0123456789abcdef' for char in etag)):
            # It looks like an MD5 hash
            if md5_digest.hexdigest() != etag:
                self.log.error('rejecting download due to ETag MD5 mismatch '
                               '(expected: %s, actual: %s)',
                               etag, md5_digest.hexdigest())
                raise RuntimeError('downloaded file appears to be corrupt '
                                   '(expected MD5: {0}, actual: {1})'
                                   .format(etag, md5_digest.hexdigest()))

        return {self.args['source']: {'md5': md5_digest.hexdigest(),
                                      'sha1': sha_digest.hexdigest(),
                                      'size': bytes_written}}



# Copyright 2014-2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from requestbuilder import Arg, MutuallyExclusiveArgList

from euca2ools.commands.ec2 import EC2Request


class CreateNetworkInterface(EC2Request):
    DESCRIPTION = 'Create a new VPC network interface'
    ARGS = [Arg('SubnetId', metavar='SUBNET', help='''subnet to create
                the new network interface in (required)'''),
            Arg('-d', '--description', dest='Description', metavar='DESC',
                help='description for the new network interface'),
            Arg('-g', '--group', dest='SecurityGroupId', metavar='GROUP',
                action='append',
                help='''ID of a security group to add the new network interface
                to.  This option may be used more than once.  Each time adds
                the network interface to an additional security group.'''),
            Arg('--private-ip-address', metavar='ADDRESS', route_to=None,
                help='''assign a specific primary private IP address to the
                new network interface'''),
            MutuallyExclusiveArgList(
                Arg('--secondary-address', '--secondary-private-ip-address',
                    metavar='ADDRESS', route_to=None, action='append',
                    help='''assign a specific secondary private IP address
                    to the new network interface.  Use this option multiple
                    times to add additional addresses.'''),
                Arg('--secondary-count', '--secondary-private-ip-address-count',
                    dest='SecondaryPrivateIpAddressCount', metavar='COUNT',
                    type=int,
                    help='''automatically assign a specific number of secondary
                    private IP addresses to the new network interface'''))]
    LIST_TAGS = ['groupSet', 'privateIpAddressesSet']

    def preprocess(self):
        addrs = []
        if self.args.get('private_ip_address'):
            addrs.append({'PrivateIpAddress': self.args['private_ip_address'],
                          'Primary': True})
        for addr in self.args.get('secondary_private_ip_address') or []:
            addrs.append({'PrivateIpAddress': addr, 'Primary': False})
        self.params['PrivateIpAddresses'] = addrs

    def print_result(self, result):
        self.print_interface(result.get('networkInterface') or {})



# Copyright 2014-2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import json

from requestbuilder import Arg, MutuallyExclusiveArgList

from euca2ools.commands.argtypes import file_contents
from euca2ools.commands.iam import IAMRequest, AS_ACCOUNT, arg_role


class UpdateAssumeRolePolicy(IAMRequest):
    DESCRIPTION = ("Update a role's trust policy, the policy that allows "
                   "entities to assume a role")
    ARGS = [arg_role(help='role to update (required)'),
            MutuallyExclusiveArgList(
                Arg('-f', dest='PolicyDocument', metavar='FILE',
                    type=file_contents,
                    help='file containing the policy for the new role'),
                Arg('-s', '--service', route_to=None, help='''service to allow
                    access to the role (e.g. ec2.amazonaws.com)'''))
            .required(),
            Arg('-o', dest='verbose', action='store_true',
                help="also print the role's new policy"),
            AS_ACCOUNT]

    def preprocess(self):
        if self.args.get('service'):
            statement = {'Effect': 'Allow',
                         'Principal': {'Service': [self.args['service']]},
                         'Action': ['sts:AssumeRole']}
            policy = {'Version': '2008-10-17',
                      'Statement': [statement]}
            self.params['PolicyDocument'] = json.dumps(policy)

    def print_result(self, _):
        if self.args.get('verbose'):
            print self.params['PolicyDocument']



# Copyright 2013 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import argparse
from euca2ools.commands.autoscaling import AutoScalingRequest
from requestbuilder import Arg


class DeletePolicy(AutoScalingRequest):
    DESCRIPTION = 'Delete a scaling policy'
    ARGS = [Arg('PolicyName', metavar='POLICY',
                help='name of the policy to delete (required)'),
            Arg('-g', '--auto-scaling-group', dest='AutoScalingGroupName',
                metavar='ASGROUP', required=True,
                help='''name of the auto-scaling group the policy is associated
                with (required)'''),
            Arg('-f', '--force', route_to=None, action='store_true',
                help=argparse.SUPPRESS)]  # for compatibility



# Copyright 2013 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from euca2ools.commands.autoscaling.argtypes import autoscaling_tag_def
from requestbuilder import Arg


class TagArg(Arg):
    def __init__(self, required=False):
        helpstr = '''attributes of a tag to affect.  Tags follow the following
        format: "id=resource-name, t=resource-type, k=tag-key, v=tag-val,
        p=propagate-at-launch-flag", where k is the tag's name, v is the tag's
        value, id is a resource ID, t is a resource type, and p is whether to
        propagate tags to instances created by the group.  A value for 'k=' is
        required for each tag.  The rest are optional.  This argument may be
        used more than once.  Each time affects a different tag.'''

        if required:
            helpstr += '  (at least 1 required)'

        Arg.__init__(self, '--tag', dest='Tags.member', required=required,
                     action='append', type=autoscaling_tag_def, help=helpstr,
                     metavar=('"k=VALUE, id=VALUE, t=VALUE, v=VALUE, '
                              'p={true,false}"'))



# Copyright 2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from requestbuilder import Arg
from requestbuilder.mixins.formatting import TableOutputMixin

from euca2ools.commands.elasticloadbalancing import ELBRequest


class DescribeTags(ELBRequest, TableOutputMixin):
    DESCRIPTION = 'Show the tags associated with one or more load balancers'
    ARGS = [Arg('LoadBalancerNames.member', metavar='ELB', nargs='+',
                help='load balancer(s) to show tags for (required)')]
    LIST_TAGS = ['TagDescriptions', 'Tags']

    def print_result(self, result):
        table = self.get_table(('TAG', 'type', 'name', 'key', 'value'))
        for desc in result.get('TagDescriptions') or []:
            lb_name = desc.get('LoadBalancerName')
            for tag in desc.get('Tags') or []:
                table.add_row(('TAG', 'load-balancer', lb_name,
                               tag.get('Key'), tag.get('Value')))
        print table



# Copyright 2013-2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import argparse

from requestbuilder import Arg, MutuallyExclusiveArgList

from euca2ools.commands.argtypes import delimited_list
from euca2ools.commands.monitoring import CloudWatchRequest
from euca2ools.commands.monitoring.argtypes import cloudwatch_dimension


def _statistic_set(set_as_str):
    pairs = {}
    for pair in set_as_str.split(','):
        try:
            key, val = pair.split('=')
        except ValueError:
            raise argparse.ArgumentTypeError(
                'statistic set must have format KEY1=VALUE1,...')
        try:
            pairs[key] = float(val)
        except ValueError:
            raise argparse.ArgumentTypeError('value "{0}" must be numeric'
                                             .format(val))
    for field in ('Maximum', 'Minimum', 'SampleCount', 'Sum'):
        if field not in pairs:
            raise argparse.ArgumentTypeError(
                'value for statistic "{0}" is required'.format(field))
    return pairs


class PutMetricData(CloudWatchRequest):
    DESCRIPTION = 'Add data points or statistics to a metric'
    ARGS = [Arg('-m', '--metric-name', dest='MetricData.member.1.MetricName',
                metavar='METRIC', required=True,
                help='name of the metric to add data points to (required)'),
            Arg('-n', '--namespace', dest='Namespace', required=True,
                help="the metric's namespace (required)"),
            MutuallyExclusiveArgList(
                Arg('-v', '--value', dest='MetricData.member.1.Value',
                    metavar='FLOAT', type=float,
                    help='data value for the metric'),
                Arg('-s', '--statistic-values', '--statisticValues',
                    dest='MetricData.member.1.StatisticValues',
                    metavar=('Maximum=FLOAT,Minimum=FLOAT,SampleCount=FLOAT,'
                             'Sum=FLOAT'), type=_statistic_set,
                    help='''statistic values for the metric.  Maximum, Minimum,
                    SampleCount, and Sum values are all required.'''))
            .required(),
            Arg('-d', '--dimensions', dest='MetricData.member.1.Dimensions.member',
                metavar='KEY1=VALUE1,KEY2=VALUE2,...',
                type=delimited_list(',', item_type=cloudwatch_dimension),
                help='the dimensions of the metric to add data points to'),
            Arg('-t', '--timestamp', dest='MetricData.member.1.Timestamp',
                metavar='YYYY-MM-DDThh:mm:ssZ',
                help='timestamp of the data point'),
            Arg('-u', '--unit', dest='MetricData.member.1.Unit',
                metavar='UNIT', help='unit the metric is being reported in')]



# Copyright 2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from requestbuilder import Arg, MutuallyExclusiveArgList

from euca2ools.commands.ec2 import EC2Request


class DescribeSnapshotAttribute(EC2Request):
    DESCRIPTION = 'Show information about an attribute of a snapshot'
    ARGS = [Arg('SnapshotId', metavar='SNAPSHOT', help='snapshot to describe'),
            MutuallyExclusiveArgList(
                Arg('-c', '--create-volume-permission', dest='Attribute',
                    action='store_const', const='createVolumePermission',
                    help='display who can create volumes from the snapshot'),
                Arg('-p', '--product-codes', dest='Attribute',
                    action='store_const', const='productCodes',
                    help='list associated product codes'))
            .required()]
    LIST_TAGS = ['createVolumePermission', 'productCodes']

    def print_result(self, result):
        snapshot_id = result.get('snapshotId')
        for perm in result.get('createVolumePermission', []):
            for (entity_type, entity_name) in perm.items():
                print self.tabify(('createVolumePermission', snapshot_id,
                                   entity_type, entity_name))
        for code in result.get('productCodes', []):
            if 'type' in code:
                code_str = '[{0}: {1}]'.format(code['type'],
                                               code.get('productCode'))
            else:
                code_str = code.get('productCode')
            print self.tabify(('productCodes', snapshot_id, 'productCode',
                               code_str))



# Copyright 2013 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from euca2ools.commands.autoscaling import AutoScalingRequest
from requestbuilder import Arg
from requestbuilder.mixins import TabifyingMixin
from requestbuilder.response import PaginatedResponse


class DescribeLaunchConfigurations(AutoScalingRequest, TabifyingMixin):
    DESCRIPTION = 'Describe auto-scaling instance launch configurations'
    ARGS = [Arg('LaunchConfigurationNames.member', metavar='LAUNCHCONFIG',
                nargs='*',
                help='limit results to specific launch configurations'),
            Arg('--show-long', action='store_true', route_to=None,
                help="show all of the launch configurations' info")]
    LIST_TAGS = ['LaunchConfigurations', 'SecurityGroups',
                 'BlockDeviceMappings']

    def main(self):
        return PaginatedResponse(self, (None,), ('LaunchConfigurations',))

    def prepare_for_page(self, page):
        # Pages are defined by NextToken
        self.params['NextToken'] = page

    def get_next_page(self, response):
        return response.get('NextToken') or None

    def print_result(self, result):
        for config in result.get('LaunchConfigurations', []):
            bits = ['LAUNCH-CONFIG',
                    config.get('LaunchConfigurationName'),
                    config.get('ImageId'),
                    config.get('InstanceType')]
            if self.args['show_long']:
                bits.append(config.get('KeyName'))
                bits.append(config.get('KernelId'))
                bits.append(config.get('RamdiskId'))
                block_maps = [convert_block_mapping_to_str(mapping) for mapping
                              in config.get('BlockDeviceMappings', [])]
                if len(block_maps) > 0:
                    bits.append('{' + ','.join(block_maps) + '}')
                else:
                    bits.append(None)
                bits.append(','.join(config.get('SecurityGroups', [])) or None)
                bits.append(config.get('CreatedTime'))
                bits.append(config.get('InstanceMonitoring', {}).get(
                    'Enabled'))
                bits.append(config.get('LaunchConfigurationARN'))
            bits.append(config.get('SpotPrice'))
            bits.append(config.get('IamInstanceProfile'))
            if self.args['show_long']:
                bits.append(config.get('EbsOptimized'))
                bits.append(config.get('AssociatePublicIpAddress'))
            print self.tabify(bits)


def convert_block_mapping_to_str(mapping):
    if mapping.get('Ebs'):
        mapped = ':'.join((mapping['Ebs'].get('SnapshotId') or '',
                           mapping['Ebs'].get('VolumeSize') or ''))
    elif mapping.get('VirtualName'):
        mapped = mapping['VirtualName']
    else:
        raise ValueError('unexpected block device mapping: {0}'.format(
            mapping))
    return mapping['DeviceName'] + '=' + mapped



# Copyright 2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import argparse

from requestbuilder import Arg
from requestbuilder.exceptions import ArgumentError, ServerError

from euca2ools.commands.s3 import S3, S3Request
from euca2ools.commands.s3.checkbucket import CheckBucket
from euca2ools.commands.s3.createbucket import CreateBucket


class S3AccessMixin(object):
    ARGS = [Arg('--s3-url', metavar='URL', route_to=None,
                help='object storage service endpoint URL'),
            Arg('-o', '--owner-akid', metavar='KEY_ID', route_to=None,
                help='''access key to use for the object storage service
                (default: same as that for the compute service)'''),
            Arg('-w', '--owner-sak', metavar='KEY', route_to=None,
                help='''secret key to use for the object storage service
                (default: same as that for the compute service)'''),
            # Pass-throughs
            Arg('--s3-service', route_to=None, help=argparse.SUPPRESS),
            Arg('--s3-auth', route_to=None, help=argparse.SUPPRESS)]

    def configure_s3_access(self):
        if self.args.get('owner_akid') and not self.args.get('owner_sak'):
            raise ArgumentError('argument -o/--owner-akid also requires '
                                '-w/--owner-sak')
        if self.args.get('owner_sak') and not self.args.get('owner_akid'):
            raise ArgumentError('argument -w/--owner-sak also requires '
                                '-o/--owner-akid')
        if not self.args.get('s3_auth'):
            if self.args.get('owner_sak') and self.args.get('owner_akid'):
                self.args['s3_auth'] = S3Request.AUTH_CLASS.from_other(
                    self.auth, key_id=self.args['owner_akid'],
                    secret_key=self.args['owner_sak'])
            else:
                self.args['s3_auth'] = S3Request.AUTH_CLASS.from_other(
                    self.auth)
        if not self.args.get('s3_service'):
            self.args['s3_service'] = S3.from_other(
                self.service, url=self.args.get('s3_url'))

    def ensure_bucket_exists(self, bucket):
        try:
            # Ensure the bucket exists
            req = CheckBucket.from_other(self, service=self.args['s3_service'],
                                         auth=self.args['s3_auth'],
                                         bucket=bucket)
            req.main()
        except ServerError as err:
            if err.status_code == 404:
                # No such bucket
                self.log.info("creating bucket: '%s'", bucket)
                req = CreateBucket.from_other(req, bucket=bucket)
                req.main()
            else:
                raise



# Copyright 2013 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from euca2ools.commands.argtypes import delimited_list
from euca2ools.commands.elasticloadbalancing import ELBRequest
from requestbuilder import Arg
from requestbuilder.mixins import TabifyingMixin


def instance_id(inst_as_str):
    return {'InstanceId': inst_as_str}


class DescribeInstanceHealth(ELBRequest, TabifyingMixin):
    DESCRIPTION = 'Show the state of instances registered with a load balancer'
    ARGS = [Arg('LoadBalancerName', metavar='ELB', help='''name of the load
                balancer to describe instances for (required)'''),
            Arg('--instances', dest='Instances.member',
                metavar='INSTANCE1,INSTANCE2,...',
                type=delimited_list(',', item_type=instance_id),
                help='limit results to specific instances'),
            Arg('--show-long', action='store_true', route_to=None,
                help="show all of the instances' info")]
    LIST_TAGS = ['InstanceStates']

    def print_result(self, result):
        for instance in result.get('InstanceStates', []):
            bits = ['INSTANCE', instance.get('InstanceId'),
                    instance.get('State')]
            if self.args['show_long']:
                bits.append(instance.get('Description'))
                bits.append(instance.get('ReasonCode'))
            print self.tabify(bits)



# Copyright 2013 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from euca2ools.commands.elasticloadbalancing import ELBRequest
from requestbuilder import Arg
from requestbuilder.mixins import TabifyingMixin


class DescribeLoadBalancers(ELBRequest, TabifyingMixin):
    DESCRIPTION = 'Show information about load balancers'
    ARGS = [Arg('LoadBalancerNames.member', metavar='ELB', nargs='*',
                help='limit results to specific load balancers'),
            Arg('--show-long', action='store_true', route_to=None,
                help="show all of the load balancers' info")]
    LIST_TAGS = ['LoadBalancerDescriptions', 'AvailabilityZones',
                 'BackendServerDescriptions', 'Instances',
                 'ListenerDescriptions', 'PolicyNames',
                 'AppCookieStickinessPolicies', 'LBCookieStickinessPolicies',
                 'OtherPolicies', 'SecurityGroups', 'Subnets']

    def print_result(self, result):
        for desc in result.get('LoadBalancerDescriptions', []):
            bits = ['LOAD_BALANCER',
                    desc.get('LoadBalancerName'),
                    desc.get('DNSName')]
            if self.args['show_long']:
                bits.append(desc.get('CanonicalHostedZoneName'))
                bits.append(desc.get('CanonicalHostedZoneNameID'))
                check = desc.get('HealthCheck')
                if check is not None:
                    check_str_bits = []
                    elem_map = (('interval', 'Interval'),
                                ('target', 'Target'),
                                ('timeout', 'Timeout'),
                                ('healthy-threshold', 'HealthyThreshold'),
                                ('unhealthy-threshold', 'UnhealthyThreshold'))
                    for name, xmlname in elem_map:
                        if check.get(xmlname):
                            check_str_bits.append(name + '=' + check[xmlname])
                    if len(check_str_bits) > 0:
                        bits.append('{' + ','.join(check_str_bits) + '}')
                    else:
                        bits.append(None)
                else:
                    bits.append(None)
                bits.append(','.join(zone for zone in
                                     desc.get('AvailabilityZones', [])))
                bits.append(','.join(net for net in desc.get('Subnets', [])))
                bits.append(desc.get('VPCId'))
                bits.append(','.join(instance.get('InstanceId') for instance in
                                     desc.get('Instances', [])))

                listeners = []
                for listenerdesc in desc.get('ListenerDescriptions', []):
                    listener = listenerdesc.get('Listener', {})
                    listener_str_bits = []
                    elem_map = (('protocol', 'Protocol'),
                                ('lb-port', 'LoadBalancerPort'),
                                ('instance-protocol', 'InstanceProtocol'),
                                ('instance-port', 'InstancePort'),
                                ('cert-id', 'SSLCertificateId'))
                    for name, xmlname in elem_map:
                        if listener.get(xmlname):
                            listener_str_bits.append(name + '=' +
                                                     listener[xmlname])
                    if listenerdesc.get('PolicyNames'):
                        listener_str_bits.append(
                            '{' + ','.join(listenerdesc['PolicyNames']) + '}')
                    listeners.append('{' + ','.join(listener_str_bits) + '}')
                if len(listeners) > 0:
                    bits.append(','.join(listeners))
                else:
                    bits.append(None)

                beservers = []
                for bedesc in desc.get('BackendServerDescriptions', []):
                    beserver_str_bits = []
                    if 'InstancePort' in bedesc:
                        beserver_str_bits.append('instance-port=' +
                                                 bedesc['InstancePort'])
                    if 'PolicyNames' in bedesc:
                        policies = ','.join(policy for policy in
                                            bedesc['PolicyNames'])
                        beserver_str_bits.append('policies={' + policies + '}')
                    beservers.append('{' + ','.join(beserver_str_bits) + '}')
                if len(beservers) > 0:
                    bits.append(','.join(beservers))
                else:
                    bits.append(None)

                all_policies = desc.get('Policies') or {}

                app_policies = all_policies.get(
                    'AppCookieStickinessPolicies') or {}
                app_policy_strs = ['{{policy-name={0},cookie-name={1}}}'
                                   .format(policy.get('PolicyName'),
                                           policy.get('CookieName'))
                                   for policy in app_policies]
                bits.append(','.join(app_policy_strs) or None)

                lb_policies = all_policies.get(
                    'LBCookieStickinessPolicies') or {}
                lb_policy_strs = ['{{policy-name={0},expiration-period={1}}}'
                                  .format(policy.get('PolicyName'),
                                          policy.get('CookieExpirationPeriod'))
                                  for policy in lb_policies]
                bits.append(','.join(lb_policy_strs) or None)

                other_policies = all_policies.get('OtherPolicies') or {}
                if other_policies:
                    bits.append('{' + ','.join(other_policies) + '}')
                else:
                    bits.append(None)

                group = desc.get('SourceSecurityGroup')
                if group:
                    bits.append('{{owner-alias={0},group-name={1}}}'.format(
                        group.get('OwnerAlias', ''),
                        group.get('GroupName', '')))
                else:
                    bits.append(None)

                if desc.get('SecurityGroups'):
                    bits.append('{' + ','.join(desc['SecurityGroups']) + '}')
                else:
                    bits.append(None)
            bits.append(desc.get('CreatedTime'))
            bits.append(desc.get('Scheme'))
            print self.tabify(bits)



# Copyright (c) 2014-2016 Hewlett Packard Enterprise Development LP
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import hashlib
import subprocess

from requestbuilder import Arg
from requestbuilder.command import BaseCommand

from euca2ools.commands import Euca2ools


class GenerateKeyFingerprint(BaseCommand):
    DESCRIPTION = ('Show the fingerprint of a private key as it would appear '
                   'in the output of euca-describe-keypairs.\n\nNote that '
                   "this will differ from the key's SSH key fingerprint.")
    SUITE = Euca2ools
    ARGS = [Arg('privkey_filename', metavar='FILE',
                help='file containing the private key (required)')]

    def main(self):
        pkcs8 = subprocess.Popen(
            ('openssl', 'pkcs8', '-in', self.args['privkey_filename'],
             '-nocrypt', '-topk8', '-outform', 'DER'), stdout=subprocess.PIPE)
        privkey = pkcs8.communicate()[0]
        if pkcs8.returncode:
            raise subprocess.CalledProcessError(pkcs8.returncode, 'openssl')
        fprint = hashlib.sha1(privkey).hexdigest()
        return ':'.join(fprint[i:i+2] for i in range(0, len(fprint), 2))

    def print_result(self, fprint):
        print fprint



# Copyright 2009-2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os.path
import sys

from requestbuilder import Arg
from requestbuilder.command import BaseCommand
from requestbuilder.mixins import (FileTransferProgressBarMixin,
                                   RegionConfigurableMixin)

from euca2ools.bundle.pipes.core import (create_unbundle_pipeline,
                                         copy_with_progressbar)
from euca2ools.bundle.util import open_pipe_fileobjs
from euca2ools.commands import Euca2ools
from euca2ools.commands.argtypes import filesize


class UnbundleStream(BaseCommand, FileTransferProgressBarMixin,
                     RegionConfigurableMixin):
    DESCRIPTION = ('Recreate an image solely from its combined bundled parts '
                   'without using a manifest\n\nUsually one would want to use '
                   'euca-unbundle instead.')
    SUITE = Euca2ools
    ARGS = [Arg('-i', dest='source', metavar='FILE',
                help='file to read the bundle from (default: stdin)'),
            Arg('-o', dest='dest', metavar='FILE',
                help='file to write the unbundled image to (default: stdout)'),
            Arg('--enc-key', metavar='HEX', required=True, help='''the
                symmetric key used to encrypt the bundle (required)'''),
            Arg('--enc-iv', metavar='HEX', required=True,
                help='''the initialization vector used to encrypt the bundle
                (required)'''),
            Arg('--image-size', metavar='BYTES', type=filesize,
                help='verify the unbundled image is a certain size'),
            Arg('--sha1-digest', metavar='HEX', help='''verify the image's
                contents against a SHA1 digest from its manifest file''')]

    # noinspection PyExceptionInherit
    def configure(self):
        BaseCommand.configure(self)
        self.update_config_view()

        if not self.args.get('source') or self.args['source'] == '-':
            # We dup stdin because the multiprocessing lib closes it
            self.args['source'] = os.fdopen(os.dup(sys.stdin.fileno()))
        elif isinstance(self.args['source'], basestring):
            self.args['source'] = open(self.args['source'])
        # Otherwise, assume it is already a file object

        if not self.args.get('dest') or self.args['dest'] == '-':
            self.args['dest'] = sys.stdout
            self.args['show_progress'] = False
        elif isinstance(self.args['dest'], basestring):
            self.args['dest'] = open(self.args['dest'], 'w')
        # Otherwise, assume it is already a file object

    def main(self):
        pbar = self.get_progressbar(maxval=self.args.get('image_size'))
        unbundle_out_r, unbundle_out_w = open_pipe_fileobjs()
        unbundle_sha1_r = create_unbundle_pipeline(
            self.args['source'], unbundle_out_w, self.args['enc_key'],
            self.args['enc_iv'], debug=self.debug)
        unbundle_out_w.close()
        actual_size = copy_with_progressbar(unbundle_out_r, self.args['dest'],
                                            progressbar=pbar)
        actual_sha1 = int(unbundle_sha1_r.recv(), 16)
        unbundle_sha1_r.close()

        expected_sha1 = int(self.args.get('sha1_digest') or '0', 16)
        expected_size = self.args.get('image_size')
        if expected_sha1 and expected_sha1 != actual_sha1:
            self.log.error('rejecting unbundle due to SHA1 mismatch '
                           '(expected SHA1: %x, actual: %x)',
                           expected_sha1, actual_sha1)
            raise RuntimeError('bundle appears to be corrupt (expected SHA1: '
                               '{0:x}, actual: {1:x})'
                               .format(expected_sha1, actual_sha1))
        expected_size = self.args.get('image_size')
        if expected_size and expected_size != actual_size:
            self.log.error('rejecting unbundle due to size mismatch '
                           '(expected: %i, actual: %i)',
                           expected_size, actual_size)
            raise RuntimeError('bundle appears to be corrupt (expected size: '
                               '{0}, actual: {1})'
                               .format(expected_size, actual_size))
        return actual_sha1, actual_size



# Copyright 2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import argparse

from requestbuilder import Arg

from euca2ools.commands.elasticloadbalancing import ELBRequest


def key_value(arg_val):
    if '=' not in arg_val:
        raise argparse.ArgumentTypeError('invalid ATTR=VALUE pair: {0}'
                                         .format(arg_val))
    return dict([arg_val.split('=', 1)])


class ModifyLoadBalancerAttributes(ELBRequest):
    DESCRIPTION = "Modify a load balancer's attributes"
    ARGS = [Arg('LoadBalancerName', metavar='ELB',
                help='the load balancer to describe (required)'),
            Arg('attrs', metavar='ATTR=VALUE', nargs='+', type=key_value,
                route_to=None, help='''name and new value of each
                attribute to modify (required)''')]

    def preprocess(self):
        self.params['LoadBalancerAttributes'] = {}
        for attr in self.args['attrs']:
            self.params['LoadBalancerAttributes'].update(attr)



# Copyright 2009-2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os.path
import sys

from requestbuilder import Arg
import requestbuilder.exceptions
from requestbuilder.mixins import TabifyingMixin
import six

from euca2ools.commands.iam import IAMRequest, arg_account_name
from euca2ools.commands.iam.createaccesskey import CreateAccessKey


CLC_CRED_CHECK_FILE = '/usr/sbin/clcadmin-assume-system-credentials'


class CreateAccount(IAMRequest, TabifyingMixin):
    DESCRIPTION = '[Eucalyptus cloud admin only] Create a new account'
    ARGS = [arg_account_name(nargs='?', help='''also add an alias (name) to the
                             new account (required on eucalyptus < 4.2)'''),
            Arg('-k', '--create-accesskey', action='store_true', route_to=None,
                help='''also create an access key for the new account's
                administrator and show it'''),
            Arg('-w', '--write-config', action='store_true', route_to=None,
                help='''output access keys and region information in the
                form of a euca2ools.ini(5) configuration file instead of
                by themselves (implies -k)'''),
            Arg('-d', '--domain', route_to=None, help='''the DNS domain to
                use for region information in configuration file output
                (default: based on IAM URL)'''),
            Arg('-l', '--set-default-user', action='store_true', route_to=None,
                help='''set this user as the default user for the region
                in euca2ools.ini(5) configuration file output.  This
                option is only useful when used with -w.''')]

    def configure(self):
        try:
            IAMRequest.configure(self)
        except requestbuilder.exceptions.AuthError as err:
            if (os.path.exists(CLC_CRED_CHECK_FILE) and len(err.args) > 0 and
                    isinstance(err.args[0], six.string_types)):
                msg = ("{0}.  If a cloud controller is running, you "
                       "can assume administrator credentials with "
                       "eval `clcadmin-assume-system-credentials`")
                err.args = (msg.format(err.args[0]),) + err.args[1:]
            raise

    def postprocess(self, result):
        if self.args.get('create_accesskey') or self.args.get('write_config'):
            obj = CreateAccessKey.from_other(
                self, UserName='admin',
                DelegateAccount=result['Account']['AccountId'],
                write_config=self.args.get('write_config'),
                domain=self.args.get('domain'),
                set_default_user=self.args.get('set_default_user'))
            key_result = obj.main()
            result.update(key_result)

    def print_result(self, result):
        if self.args.get('write_config'):
            result['configfile'].write(sys.stdout)
        else:
            print self.tabify((result.get('Account', {}).get('AccountName'),
                               result.get('Account', {}).get('AccountId')))
            if 'AccessKey' in result:
                print result['AccessKey']['AccessKeyId']
                print result['AccessKey']['SecretAccessKey']



# Copyright 2009-2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import hashlib
import os
import multiprocessing

from requestbuilder import Arg
from requestbuilder.command import BaseCommand
from requestbuilder.exceptions import ArgumentError
from requestbuilder.mixins import (FileTransferProgressBarMixin,
                                   RegionConfigurableMixin)

from euca2ools.commands import Euca2ools
import euca2ools.bundle.pipes
from euca2ools.bundle.manifest import BundleManifest
from euca2ools.bundle.util import (close_all_fds, open_pipe_fileobjs,
                                   waitpid_in_thread)
from euca2ools.commands.bundle.unbundlestream import UnbundleStream


class Unbundle(BaseCommand, FileTransferProgressBarMixin,
               RegionConfigurableMixin):
    DESCRIPTION = ('Recreate an image from its bundled parts\n\nThe key used '
                   'to unbundle the image must match a certificate that was '
                   'used to bundle it.')
    SUITE = Euca2ools
    ARGS = [Arg('-m', '--manifest', type=open, metavar='FILE',
                required=True, help="the bundle's manifest file (required)"),
            Arg('-s', '--source', metavar='DIR', default='.',
                help='''directory containing the bundled image parts (default:
                current directory)'''),
            Arg('-d', '--destination', metavar='DIR', default='.',
                help='''where to place the unbundled image (default: current
                directory)'''),
            Arg('-k', '--privatekey', metavar='FILE', help='''file containing
                the private key to decrypt the bundle with.  This must match
                a certificate used when bundling the image.''')]

    # noinspection PyExceptionInherit
    def configure(self):
        BaseCommand.configure(self)
        self.update_config_view()

        # The private key could be the user's or the cloud's.  In the config
        # this is a user-level option.
        if not self.args.get('privatekey'):
            config_privatekey = self.config.get_user_option('private-key')
            if self.args.get('userregion'):
                self.args['privatekey'] = config_privatekey
            elif 'EC2_PRIVATE_KEY' in os.environ:
                self.args['privatekey'] = os.getenv('EC2_PRIVATE_KEY')
            elif config_privatekey:
                self.args['privatekey'] = config_privatekey
            else:
                raise ArgumentError(
                    'missing private key; please supply one with -k')
        self.args['privatekey'] = os.path.expanduser(os.path.expandvars(
            self.args['privatekey']))
        if not os.path.exists(self.args['privatekey']):
            raise ArgumentError("private key file '{0}' does not exist"
                                .format(self.args['privatekey']))
        if not os.path.isfile(self.args['privatekey']):
            raise ArgumentError("private key file '{0}' is not a file"
                                .format(self.args['privatekey']))
        self.log.debug('private key: %s', self.args['privatekey'])

        if not os.path.exists(self.args.get('source', '.')):
            raise ArgumentError("argument -s/--source: directory '{0}' does "
                                "not exist".format(self.args['source']))
        if not os.path.isdir(self.args.get('source', '.')):
            raise ArgumentError("argument -s/--source: '{0}' is not a "
                                "directory".format(self.args['source']))
        if not os.path.exists(self.args.get('destination', '.')):
            raise ArgumentError("argument -d/--destination: directory '{0}' "
                                "does not exist"
                                .format(self.args['destination']))
        if not os.path.isdir(self.args.get('destination', '.')):
            raise ArgumentError("argument -d/--destination: '{0}' is not a "
                                "directory".format(self.args['destination']))

    def __read_bundle_parts(self, manifest, outfile):
        close_all_fds(except_fds=[outfile])
        for part in manifest.image_parts:
            self.log.debug("opening part '%s' for reading", part.filename)
            digest = hashlib.sha1()
            with open(part.filename) as part_file:
                while True:
                    chunk = part_file.read(euca2ools.BUFSIZE)
                    if chunk:
                        digest.update(chunk)
                        outfile.write(chunk)
                        outfile.flush()
                    else:
                        break
                actual_digest = int(digest.hexdigest(), 16)
                expected_digest = int(part.hexdigest, 16)
                if actual_digest != expected_digest:
                    self.log.error('rejecting unbundle due to part SHA1 '
                                   'mismatch (expected: %x, actual: %x)',
                                   expected_digest, actual_digest)
                    raise RuntimeError(
                        "bundle part '{0}' appears to be corrupt (expected "
                        "SHA1: {1:x}, actual: {2:x}"
                        .format(part.filename, expected_digest, actual_digest))

    def main(self):
        manifest = BundleManifest.read_from_fileobj(
            self.args['manifest'], privkey_filename=self.args['privatekey'])

        for part in manifest.image_parts:
            part_path = os.path.join(self.args['source'], part.filename)
            while part_path.startswith('./'):
                part_path = part_path[2:]
            if os.path.exists(part_path):
                part.filename = part_path
            else:
                raise RuntimeError(
                    "bundle part '{0}' does not exist; you may need to use "
                    "-s to specify where to find the bundle's parts"
                    .format(part_path))

        part_reader_out_r, part_reader_out_w = open_pipe_fileobjs()
        part_reader = multiprocessing.Process(
            target=self.__read_bundle_parts,
            args=(manifest, part_reader_out_w))
        part_reader.start()
        part_reader_out_w.close()
        waitpid_in_thread(part_reader.pid)

        image_filename = os.path.join(self.args['destination'],
                                      manifest.image_name)
        with open(image_filename, 'w') as image:
            unbundlestream = UnbundleStream.from_other(
                self, source=part_reader_out_r, dest=image,
                enc_key=manifest.enc_key, enc_iv=manifest.enc_iv,
                image_size=manifest.image_size,
                sha1_digest=manifest.image_digest,
                show_progress=self.args.get('show_progress', False))
            unbundlestream.main()
        return image_filename

    def print_result(self, image_filename):
        print 'Wrote', image_filename



# Copyright 2013 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from euca2ools.commands.ec2 import EC2Request
from requestbuilder import Arg


class CopyImage(EC2Request):
    DESCRIPTION = ('Copy an image from another region\n\nRun this command '
                   'against the destination region, not the source region.')
    ARGS = [Arg('-r', '--source-region', dest='SourceRegion', metavar='REGION',
                required=True,
                help='name of the region to copy the image from (required)'),
            Arg('-s', '--source-ami-id', dest='SourceImageId', metavar='IMAGE',
                required=True,
                help='ID of the image to copy (required)'),
            Arg('-n', '--name', dest='Name',
                help='name to assign the new copy of the image'),
            Arg('-d', '--description', dest='Description', metavar='DESC',
                help='description to assign the new copy of the image'),
            Arg('-c', '--client-token', dest='ClientToken', metavar='TOKEN',
                help='unique identifier to ensure request idempotency')]

    def print_result(self, result):
        print self.tabify(('IMAGE', result.get('imageId')))



# Copyright 2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import sys

from requestbuilder import Arg

from euca2ools.commands.ec2 import EC2Request


class CreateVpnConnection(EC2Request):
    DESCRIPTION = ('Create a VPN connection between a virtual private '
                   'gateway and a customer gateway\n\nYou can optionally '
                   'format the connection information for specific '
                   'devices using the --format or --stylesheet options.  '
                   'If the --stylesheet option is an HTTP or HTTPS URL it '
                   'will be downloaded as needed.')
    ARGS = [Arg('-t', '--type', dest='Type', metavar='ipsec.1', required=True,
                choices=('ipsec.1',),
                help='the type of VPN connection to use (required)'),
            Arg('--customer-gateway', dest='CustomerGatewayId', required=True,
                metavar='CGATEWAY',
                help='ID of the customer gateway to connect (required)'),
            Arg('--vpn-gateway', dest='VpnGatewayId', required=True,
                metavar='VGATEWAY',
                help='ID of the virtual private gateway to connect (required)'),
            Arg('--static-routes-only', dest='Options.StaticRoutesOnly',
                action='store_true',
                help='use only static routes instead of BGP'),
            Arg('--format', route_to=None, help='''show connection
                information in a specific format (cisco-ios-isr,
                juniper-junos-j, juniper-screenos-6.1, juniper-screenos-6.2,
                generic, xml, none) (default: xml)'''),
            Arg('--stylesheet', route_to=None, help='''format the connection
                information using an XSL stylesheet.  If the value contains
                "{format}" it will be replaced with the format chosen by the
                --format option.  If the value is an HTTP or HTTPS URL it
                will be downloaded as needed.  (default: value of
                "vpn-stylesheet" region option)''')]

    def print_result(self, result):
        if self.args.get('format') is None:
            # If --stylesheet is used it will be applied.  Otherwise,
            # None will make it print the raw XML, which is what we want.
            stylesheet = self.args.get('stylesheet')
            show_conn_info = True
        elif self.args.get('format') == 'none':
            stylesheet = None
            show_conn_info = False
        elif self.args.get('format') == 'xml':
            stylesheet = None
            show_conn_info = True
        else:
            stylesheet = self.args.get('stylesheet')
            if not stylesheet:
                stylesheet = self.config.get_region_option('vpn-stylesheet')
            if stylesheet:
                stylesheet = stylesheet.format(format=self.args['format'])
            else:
                self.log.warn('current region has no stylesheet')
                msg = ('current region has no XSLT stylesheet to format '
                       'output; connection info will not be shown  (try '
                       'specifying one with "--stylesheet" or using '
                       '"--format xml")')
                print >> sys.stderr, msg
            show_conn_info = bool(stylesheet)
        self.print_vpn_connection(result.get('vpnConnection') or {},
                                  show_conn_info=show_conn_info,
                                  stylesheet=stylesheet)



# Copyright 2009-2015 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from requestbuilder import Arg, MutuallyExclusiveArgList

from euca2ools.commands.iam import IAMRequest, AS_ACCOUNT, arg_server_cert


class UploadServerCertificate(IAMRequest):
    DESCRIPTION = 'Upload a server certificate'
    ARGS = [arg_server_cert(
                help='name to give the new server certificate (required)'),
            MutuallyExclusiveArgList(
                Arg('-c', '--certificate-body', dest='CertificateBody',
                    metavar='CERT_CONTENT', help='PEM-encoded certificate'),
                Arg('--certificate-file', dest='CertificateBody',
                    metavar='FILE', type=open,
                    help='file containing the PEM-encoded certificate'))
            .required(),
            MutuallyExclusiveArgList(
                Arg('--private-key', dest='PrivateKey', metavar='KEY',
                    help='PEM-encoded private key'),
                Arg('--private-key-file', dest='PrivateKey', metavar='FILE',
                    type=open,
                    help='file containing the PEM-encoded private key'))
            .required(),
            MutuallyExclusiveArgList(
                Arg('--certificate-chain', dest='CertificateChain',
                    metavar='CHAIN', help='''PEM-encoded certificate chain.
                    This is typically the PEM-encoded certificates of the
                    chain, concatenated together.'''),
                Arg('--certificate-chain-file', dest='CertificateChain',
                    metavar='FILE', type=open,
                    help='''file containing the PEM-encoded
                    certificate chain. This is typically the PEM-encoded
                    certificates of the chain, concatenated together.''')),
            Arg('-p', '--path', dest='Path',
                help='path for the new server certificate (default: "/")'),
            AS_ACCOUNT]



# Copyright 2013-2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from requestbuilder import Arg, Filter, GenericTagFilter

from euca2ools.commands.ec2 import EC2Request


class DescribeRouteTables(EC2Request):
    DESCRIPTION = 'Describe one or more VPC route tables'
    ARGS = [Arg('RouteTableId', metavar='RTABLE', nargs='*',
                help='limit results to specific route tables')]
    FILTERS = [Filter('association.route-table-association-id',
                      help='ID of an association for the route table'),
               Filter('association.route-table-id',
                      help='ID of a route table involved in an association'),
               Filter('association.subnet-id',
                      help='ID of a subnet involved in an association'),
               Filter('association.main', choices=('true', 'false'),
                      help='''whether the route table is the main route
                      table for its VPC'''),
               Filter('route-table-id'),
               Filter('route.destination-cidr-block', help='''CIDR address
                      block specified in one of the table's routes'''),
               Filter('route.gateway-id', help='''ID of a gateway
                      specified by a route in the table'''),
               Filter('route.instance-id', help='''ID of an instance
                      specified by a route in the table'''),
               Filter('route.vpc-peering-connection-id',
                      help='''ID of a VPC peering connection specified
                      by a route in the table'''),
               Filter('route.origin',
                      help='which operation created a route in the table'),
               Filter('route.state', help='''whether a route in the
                      table has state "active" or "blackhole"'''),
               Filter('tag-key',
                      help='key of a tag assigned to the route table'),
               Filter('tag-value',
                      help='value of a tag assigned to the route table'),
               GenericTagFilter('tag:KEY',
                                help='specific tag key/value combination'),
               Filter('vpc-id', help="the associated VPC's ID")]

    LIST_TAGS = ['associationSet', 'propagatingVgwSet', 'routeTableSet',
                 'routeSet', 'tagSet']

    def print_result(self, result):
        for table in result.get('routeTableSet') or []:
            self.print_route_table(table)



# Copyright 2013-2014 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from euca2ools.commands.s3 import S3Request
from requestbuilder import Arg
from requestbuilder.mixins import TabifyingMixin
from requestbuilder.xmlparse import parse_listdelimited_aws_xml


class ListAllMyBuckets(S3Request, TabifyingMixin):
    DESCRIPTION = 'List all buckets owned by your account'
    ARGS = [Arg('-l', dest='long_output', action='store_true', route_to=None,
                help='''list in long format, with creation dates and owner
                info'''),
            Arg('-n', dest='numeric_output', action='store_true',
                route_to=None, help='''display account IDs numerically in long
                (-l) output.  This option turns on the -l option.''')]

    def configure(self):
        S3Request.configure(self)
        if self.args['numeric_output']:
            self.args['long_output'] = True

    def preprocess(self):
        self.method = 'GET'
        self.path = ''

    def parse_response(self, response):
        response_dict = self.log_and_parse_response(
            response, parse_listdelimited_aws_xml, list_tags=('Buckets',))
        return response_dict['ListAllMyBucketsResult']

    def print_result(self, result):
        if self.args['numeric_output'] or 'DisplayName' not in result['Owner']:
            owner = result.get('Owner', {}).get('ID')
        else:
            owner = result.get('Owner', {}).get('DisplayName')

        for bucket in result.get('Buckets', []):
            if self.args['long_output']:
                print self.tabify((owner, bucket.get('CreationDate'),
                                   bucket.get('Name')))
            else:
                print bucket.get('Name')
